@@ -13,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors:
  *	Vincent Untz <vuntz@gnome.org>
@@ -481,12 +483,12 @@ option_parse (const char  *option_name,
 static gboolean
 set_mode (int         new_mode,
           const char *option,
-          gboolean    use_list)
+          gboolean    list)
 {
   switch (mode)
     {
       case INVALID_MODE:
-        if (use_list)
+        if (list)
           g_assert_not_reached ();
 
         mode = new_mode;
@@ -494,7 +496,7 @@ set_mode (int         new_mode,
       case SCREEN_READ_MODE:
         if (new_mode == SCREEN_READ_MODE || new_mode == SCREEN_WRITE_MODE)
           mode = new_mode;
-        else if (use_list)
+        else if (list)
           mode = SCREEN_LIST_MODE;
         else
           {
@@ -526,7 +528,7 @@ set_mode (int         new_mode,
       case WORKSPACE_READ_MODE:
         if (new_mode == WORKSPACE_READ_MODE || new_mode == WORKSPACE_WRITE_MODE)
           mode = new_mode;
-        else if (use_list)
+        else if (list)
           mode = WORKSPACE_LIST_MODE;
         else
           {
@@ -558,7 +560,7 @@ set_mode (int         new_mode,
           }
         break;
       case APPLICATION_READ_MODE:
-        if (use_list)
+        if (list)
           mode = APPLICATION_LIST_MODE;
         else if (new_mode != APPLICATION_READ_MODE)
           {
@@ -580,7 +582,7 @@ set_mode (int         new_mode,
           }
         break;
       case CLASS_GROUP_READ_MODE:
-        if (use_list)
+        if (list)
           mode = CLASS_GROUP_LIST_MODE;
         else if (new_mode != CLASS_GROUP_READ_MODE)
           {
@@ -1673,7 +1675,6 @@ static gboolean
 wm_state_set (Display *display,
               Window   window)
 {
-  GdkDisplay *gdk_display;
   Atom    wm_state;
   gulong  nitems;
   gulong  bytes_after;
@@ -1682,19 +1683,16 @@ wm_state_set (Display *display,
   int     ret_format;
   int     err, result;
 
-  gdk_display = gdk_x11_lookup_xdisplay (display);
-  g_assert (gdk_display != NULL);
-
   wm_state = gdk_x11_get_xatom_by_name ("WM_STATE");
 
-  gdk_x11_display_error_trap_push (gdk_display);
+  gdk_error_trap_push ();
   result = XGetWindowProperty (display,
                                window,
                                wm_state,
                                0, G_MAXLONG,
                                False, wm_state, &ret_type, &ret_format, &nitems,
                                &bytes_after, (gpointer) &prop);
-  err = gdk_x11_display_error_trap_pop (gdk_display);
+  err = gdk_error_trap_pop ();
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -1711,23 +1709,19 @@ static WnckWindow *
 find_managed_window (Display *display,
                      Window   window)
 {
-  GdkDisplay *gdk_display;
   Window      root;
   Window      parent;
   Window     *kids = NULL;
   WnckWindow *retval;
-  guint       i, nkids;
-  int         result;
+  guint       nkids;
+  int         i, result;
 
   if (wm_state_set (display, window))
     return wnck_window_get (window);
 
-  gdk_display = gdk_x11_lookup_xdisplay (display);
-  g_assert (gdk_display != NULL);
-
-  gdk_x11_display_error_trap_push (gdk_display);
+  gdk_error_trap_push ();
   result = XQueryTree (display, window, &root, &parent, &kids, &nkids);
-  if (gdk_x11_display_error_trap_pop (gdk_display) || !result)
+  if (gdk_error_trap_pop () || !result)
     return NULL;
 
   retval = NULL;
@@ -1762,7 +1756,7 @@ handle_button_press_event (Display *dpy, XIDeviceEvent *event)
 
 static GdkFilterReturn
 target_filter (GdkXEvent *gdk_xevent,
-               GdkEvent  *gdk_event,
+               GdkEvent  *event,
                gpointer   data)
 {
   XEvent *xevent = (XEvent *) gdk_xevent;
@@ -1797,43 +1791,61 @@ target_filter (GdkXEvent *gdk_xevent,
   return GDK_FILTER_CONTINUE;
 }
 
-static void
-prepare (GdkSeat   *seat,
-         GdkWindow *window,
-         gpointer   user_data)
-{
-  gdk_window_show_unraised (window);
-}
-
 static gboolean
 get_target (gpointer data)
 {
-  GdkWindow *root;
-  GdkDisplay *display;
-  GdkSeat *seat;
-  GdkCursor *cross;
-  GdkSeatCapabilities caps;
-  GdkGrabStatus status;
+  GdkGrabStatus     status;
+  GdkDeviceManager *dev_manager;
+  GdkDevice        *device;
+  GdkCursor        *cross;
+  GdkWindow        *root;
+  GList            *devices, *l;
 
+  dev_manager = gdk_display_get_device_manager (gdk_display_get_default ());
   root = gdk_get_default_root_window ();
-  display = gdk_display_get_default ();
-  seat = gdk_display_get_default_seat (display);
-  cross = gdk_cursor_new_for_display (display, GDK_CROSS);
-  caps = GDK_SEAT_CAPABILITY_POINTER | GDK_SEAT_CAPABILITY_KEYBOARD;
 
   gdk_window_add_filter (root, (GdkFilterFunc) target_filter, NULL);
 
-  status = gdk_seat_grab (seat, root, caps, TRUE, cross, NULL, prepare, NULL);
+  cross = gdk_cursor_new (GDK_CROSS);
+  device = gdk_device_manager_get_client_pointer (dev_manager);
+  status = gdk_device_grab (device, root, GDK_OWNERSHIP_WINDOW, TRUE,
+                            GDK_BUTTON_PRESS_MASK, cross, GDK_CURRENT_TIME);
   g_object_unref (cross);
 
   if (status != GDK_GRAB_SUCCESS)
     {
-      g_warning ("Seat grab failed.");
+      g_warning ("Pointer grab failed.\n");
       clean_up ();
       return FALSE;
     }
 
-  gdk_display_flush (display);
+  devices = gdk_device_manager_list_devices (dev_manager, GDK_DEVICE_TYPE_MASTER);
+
+  for (l = devices; l; l = l->next)
+    {
+      device = GDK_DEVICE (l->data);
+
+      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
+        continue;
+
+      status = gdk_device_grab (device, root, GDK_OWNERSHIP_NONE, TRUE,
+                                GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK, NULL,
+                                GDK_CURRENT_TIME);
+
+      if (status != GDK_GRAB_SUCCESS)
+        break;
+    }
+
+  g_list_free (devices);
+
+  if (status != GDK_GRAB_SUCCESS)
+    {
+      g_warning ("Keyboard grab failed.\n");
+      clean_up ();
+      return FALSE;
+    }
+
+  gdk_flush ();
 
   return FALSE;
 }
@@ -1842,15 +1854,30 @@ static void
 clean_up (void)
 {
   GdkWindow *root;
-  GdkDisplay *display;
-  GdkSeat *seat;
+  GdkDeviceManager *dev_manager;
+  GdkDevice        *device;
+  GList            *devices, *l;
 
   root = gdk_get_default_root_window ();
-  display = gdk_display_get_default ();
-  seat = gdk_display_get_default_seat (display);
-
+  dev_manager = gdk_display_get_device_manager (gdk_display_get_default ());
   gdk_window_remove_filter (root, (GdkFilterFunc) target_filter, NULL);
-  gdk_seat_ungrab (seat);
+
+  device = gdk_device_manager_get_client_pointer (dev_manager);
+  gdk_device_ungrab (device, GDK_CURRENT_TIME);
+
+  devices = gdk_device_manager_list_devices (dev_manager, GDK_DEVICE_TYPE_MASTER);
+
+  for (l = devices; l; l = l->next)
+    {
+      device = GDK_DEVICE (l->data);
+
+      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
+        continue;
+
+      gdk_device_ungrab (device, GDK_CURRENT_TIME);
+    }
+
+  g_list_free (devices);
 
   gtk_main_quit ();
 }
