@@ -16,7 +16,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <config.h>
@@ -24,12 +26,10 @@
 #include <string.h>
 #include <stdio.h>
 #include <cairo-xlib.h>
-#if HAVE_CAIRO_XLIB_XRENDER
-#include <cairo-xlib-xrender.h>
-#endif
 #include "screen.h"
 #include "window.h"
 #include "private.h"
+#include "inlinepixbufs.h"
 
 gboolean
 _wnck_get_cardinal (Screen *screen,
@@ -699,24 +699,14 @@ _wnck_set_utf8_list (Screen  *screen,
 void
 _wnck_error_trap_push (Display *display)
 {
-  GdkDisplay *gdk_display;
-
-  gdk_display = gdk_x11_lookup_xdisplay (display);
-  g_assert (gdk_display != NULL);
-
-  gdk_x11_display_error_trap_push (gdk_display);
+  gdk_error_trap_push ();
 }
 
 int
 _wnck_error_trap_pop (Display *display)
 {
-  GdkDisplay *gdk_display;
-
-  gdk_display = gdk_x11_lookup_xdisplay (display);
-  g_assert (gdk_display != NULL);
-
-  gdk_display_flush (gdk_display);
-  return gdk_x11_display_error_trap_pop (gdk_display);
+  XSync (display, False);
+  return gdk_error_trap_pop ();
 }
 
 static GdkFilterReturn
@@ -794,9 +784,6 @@ filter_func (GdkXEvent  *gdkxevent,
         }
 #endif /* HAVE_STARTUP_NOTIFICATION */
       break;
-
-    default:
-      break;
     }
 
   return GDK_FILTER_CONTINUE;
@@ -839,7 +826,7 @@ _wnck_xid_hash (gconstpointer v)
   gulong val = * (const gulong *) v;
 
   /* I'm not sure this works so well. */
-#if GLIB_SIZEOF_LONG > 4
+#if G_SIZEOF_LONG > 4
   return (guint) (val ^ (val >> 32));
 #else
   return val;
@@ -882,18 +869,16 @@ _wnck_deiconify (Screen *screen,
 }
 
 void
-_wnck_close (WnckScreen *screen,
-             Window      xwindow,
-             Time        timestamp)
+_wnck_close (Screen *screen,
+	     Window  xwindow,
+	     Time    timestamp)
 {
-  Screen *xscreen;
   Display *display;
-  Window root;
-  XEvent xev;
+  Window   root;
+  XEvent   xev;
 
-  xscreen = _wnck_screen_get_xscreen (screen);
-  display = DisplayOfScreen (xscreen);
-  root = RootWindowOfScreen (xscreen);
+  display = DisplayOfScreen (screen);
+  root = RootWindowOfScreen (screen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -930,17 +915,15 @@ _wnck_close (WnckScreen *screen,
 #define _NET_WM_MOVERESIZE_MOVE_KEYBOARD    10
 
 void
-_wnck_keyboard_move (WnckScreen *screen,
-                     Window      xwindow)
+_wnck_keyboard_move (Screen *screen,
+                     Window  xwindow)
 {
-  Screen *xscreen;
   Display *display;
-  Window root;
-  XEvent xev;
+  Window   root;
+  XEvent   xev;
 
-  xscreen = _wnck_screen_get_xscreen (screen);
-  display = DisplayOfScreen (xscreen);
-  root = RootWindowOfScreen (xscreen);
+  display = DisplayOfScreen (screen);
+  root = RootWindowOfScreen (screen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -965,17 +948,15 @@ _wnck_keyboard_move (WnckScreen *screen,
 }
 
 void
-_wnck_keyboard_size (WnckScreen *screen,
-                     Window      xwindow)
+_wnck_keyboard_size (Screen *screen,
+                     Window  xwindow)
 {
-  Screen *xscreen;
   Display *display;
-  Window root;
-  XEvent xev;
+  Window   root;
+  XEvent   xev;
 
-  xscreen = _wnck_screen_get_xscreen (screen);
-  display = DisplayOfScreen (xscreen);
-  root = RootWindowOfScreen (xscreen);
+  display = DisplayOfScreen (screen);
+  root = RootWindowOfScreen (screen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1000,24 +981,22 @@ _wnck_keyboard_size (WnckScreen *screen,
 }
 
 void
-_wnck_change_state (WnckScreen *screen,
-                    Window      xwindow,
-                    gboolean    add,
-                    Atom        state1,
-                    Atom        state2)
+_wnck_change_state (Screen  *screen,
+		    Window   xwindow,
+                    gboolean add,
+                    Atom     state1,
+                    Atom     state2)
 {
-  Screen *xscreen;
   Display *display;
-  Window root;
-  XEvent xev;
+  Window   root;
+  XEvent   xev;
 
 #define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
 #define _NET_WM_STATE_ADD           1    /* add/set property */
 #define _NET_WM_STATE_TOGGLE        2    /* toggle property  */
 
-  xscreen = _wnck_screen_get_xscreen (screen);
-  display = DisplayOfScreen (xscreen);
-  root = RootWindowOfScreen (xscreen);
+  display = DisplayOfScreen (screen);
+  root = RootWindowOfScreen (screen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1042,18 +1021,16 @@ _wnck_change_state (WnckScreen *screen,
 }
 
 void
-_wnck_change_workspace (WnckScreen *screen,
-                        Window      xwindow,
+_wnck_change_workspace (Screen     *screen,
+			Window      xwindow,
                         int         new_space)
 {
-  Screen *xscreen;
   Display *display;
-  Window root;
-  XEvent xev;
+  Window   root;
+  XEvent   xev;
 
-  xscreen = _wnck_screen_get_xscreen (screen);
-  display = DisplayOfScreen (xscreen);
-  root = RootWindowOfScreen (xscreen);
+  display = DisplayOfScreen (screen);
+  root = RootWindowOfScreen (screen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1078,22 +1055,20 @@ _wnck_change_workspace (WnckScreen *screen,
 }
 
 void
-_wnck_activate (WnckScreen *screen,
-                Window      xwindow,
-                Time        timestamp)
+_wnck_activate (Screen *screen,
+                Window  xwindow,
+                Time    timestamp)
 {
-  Screen *xscreen;
   Display *display;
-  Window root;
-  XEvent xev;
+  Window   root;
+  XEvent   xev;
 
   if (timestamp == 0)
     g_warning ("Received a timestamp of 0; window activation may not "
                "function properly.\n");
 
-  xscreen = _wnck_screen_get_xscreen (screen);
-  display = DisplayOfScreen (xscreen);
-  root = RootWindowOfScreen (xscreen);
+  display = DisplayOfScreen (screen);
+  root = RootWindowOfScreen (screen);
 
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1392,23 +1367,6 @@ _wnck_get_frame_extents (Screen *screen,
       retval = TRUE;
     }
 
-  if (p_size == NULL)
-    {
-      _wnck_get_cardinal_list (screen, xwindow,
-                               _wnck_atom_get ("_GTK_FRAME_EXTENTS"),
-                               &p_size, &n_size);
-
-      if (p_size != NULL && n_size == 4)
-        {
-          *left_frame   = -p_size[0];
-          *right_frame  = -p_size[1];
-          *top_frame    = -p_size[2];
-          *bottom_frame = -p_size[3];
-
-          retval = TRUE;
-        }
-    }
-
   if (p_size != NULL)
     g_free (p_size);
 
@@ -1473,7 +1431,7 @@ find_largest_sizes (gulong *data,
       w = data[0];
       h = data[1];
 
-      if (nitems < ((gulong) (w * h) + 2))
+      if (nitems < ((w * h) + 2))
         return FALSE; /* not enough data */
 
       *width = MAX (w, *width);
@@ -1529,7 +1487,7 @@ find_best_size (gulong  *data,
       w = data[0];
       h = data[1];
 
-      if (nitems < ((gulong) (w * h) + 2))
+      if (nitems < ((w * h) + 2))
         break; /* not enough data */
 
       if (best_start == NULL)
@@ -1733,34 +1691,10 @@ _wnck_cairo_surface_get_from_pixmap (Screen *screen,
       if (!XGetWindowAttributes (display, root_return, &attrs))
         goto TRAP_POP;
 
-      if (depth_ret == (unsigned int) attrs.depth)
-	{
-	  surface = cairo_xlib_surface_create (display,
-					       xpixmap,
-					       attrs.visual,
-					       w_ret, h_ret);
-	}
-      else
-	{
-#if HAVE_CAIRO_XLIB_XRENDER
-	  int std;
-
-	  switch (depth_ret) {
-	  case 1: std = PictStandardA1; break;
-	  case 4: std = PictStandardA4; break;
-	  case 8: std = PictStandardA8; break;
-	  case 24: std = PictStandardRGB24; break;
-	  case 32: std = PictStandardARGB32; break;
-	  default: goto TRAP_POP;
-	  }
-
-	  surface = cairo_xlib_surface_create_with_xrender_format (display,
-								   xpixmap,
-								   attrs.screen,
-								   XRenderFindStandardFormat (display, std),
-								   w_ret, h_ret);
-#endif
-	}
+      surface = cairo_xlib_surface_create (display,
+                                           xpixmap,
+                                           attrs.visual,
+                                           w_ret, h_ret);
     }
 
 TRAP_POP:
@@ -1803,7 +1737,6 @@ try_pixmap_and_mask (Screen     *screen,
                      int         ideal_mini_height)
 {
   cairo_surface_t *surface, *mask_surface, *image;
-  GdkDisplay *gdk_display;
   GdkPixbuf *unscaled;
   int width, height;
   cairo_t *cr;
@@ -1821,11 +1754,6 @@ try_pixmap_and_mask (Screen     *screen,
   if (surface == NULL)
     return FALSE;
 
-  gdk_display = gdk_x11_lookup_xdisplay (XDisplayOfScreen (screen));
-  g_assert (gdk_display != NULL);
-
-  gdk_x11_display_error_trap_push (gdk_display);
-
   width = cairo_xlib_surface_get_width (surface);
   height = cairo_xlib_surface_get_height (surface);
 
@@ -1864,12 +1792,6 @@ try_pixmap_and_mask (Screen     *screen,
   cairo_surface_destroy (surface);
   cairo_destroy (cr);
 
-  if (gdk_x11_display_error_trap_pop (gdk_display) != Success)
-    {
-      cairo_surface_destroy (image);
-      return FALSE;
-    }
-
   unscaled = gdk_pixbuf_get_from_surface (image,
                                           0, 0,
                                           width, height);
@@ -2159,17 +2081,16 @@ scaled_from_pixdata (guchar *pixdata,
 }
 
 gboolean
-_wnck_read_icons (WnckScreen     *screen,
-                  Window          xwindow,
-                  WnckIconCache  *icon_cache,
-                  GdkPixbuf     **iconp,
-                  int             ideal_width,
-                  int             ideal_height,
-                  GdkPixbuf     **mini_iconp,
-                  int             ideal_mini_width,
-                  int             ideal_mini_height)
+_wnck_read_icons (Screen        *screen,
+                  Window         xwindow,
+                  WnckIconCache *icon_cache,
+                  GdkPixbuf    **iconp,
+                  int            ideal_width,
+                  int            ideal_height,
+                  GdkPixbuf    **mini_iconp,
+                  int            ideal_mini_width,
+                  int            ideal_mini_height)
 {
-  Screen *xscreen;
   Display *display;
   guchar *pixdata;
   int w, h;
@@ -2183,8 +2104,7 @@ _wnck_read_icons (WnckScreen     *screen,
 
   g_return_val_if_fail (icon_cache != NULL, FALSE);
 
-  xscreen = _wnck_screen_get_xscreen (screen);
-  display = DisplayOfScreen (xscreen);
+  display = DisplayOfScreen (screen);
 
   *iconp = NULL;
   *mini_iconp = NULL;
@@ -2220,7 +2140,7 @@ _wnck_read_icons (WnckScreen     *screen,
     {
       icon_cache->net_wm_icon_dirty = FALSE;
 
-      if (read_rgb_icon (xscreen, xwindow,
+      if (read_rgb_icon (screen, xwindow,
                          ideal_width, ideal_height,
                          ideal_mini_width, ideal_mini_height,
                          &w, &h, &pixdata,
@@ -2267,7 +2187,7 @@ _wnck_read_icons (WnckScreen     *screen,
            mask != icon_cache->prev_mask) &&
           pixmap != None)
         {
-          if (try_pixmap_and_mask (xscreen, pixmap, mask,
+          if (try_pixmap_and_mask (screen, pixmap, mask,
                                    iconp, ideal_width, ideal_height,
                                    mini_iconp, ideal_mini_width, ideal_mini_height))
             {
@@ -2287,13 +2207,13 @@ _wnck_read_icons (WnckScreen     *screen,
     {
       icon_cache->kwm_win_icon_dirty = FALSE;
 
-      get_kwm_win_icon (xscreen, xwindow, &pixmap, &mask);
+      get_kwm_win_icon (screen, xwindow, &pixmap, &mask);
 
       if ((pixmap != icon_cache->prev_pixmap ||
            mask != icon_cache->prev_mask) &&
           pixmap != None)
         {
-          if (try_pixmap_and_mask (xscreen, pixmap, mask,
+          if (try_pixmap_and_mask (screen, pixmap, mask,
                                    iconp, ideal_width, ideal_height,
                                    mini_iconp, ideal_mini_width, ideal_mini_height))
             {
@@ -2341,9 +2261,12 @@ static GdkPixbuf*
 default_icon_at_size (int width,
                       int height)
 {
+
   GdkPixbuf *base;
 
-  base = gdk_pixbuf_new_from_resource ("/org/gnome/libwnck/default_icon.png", NULL);
+  base = gdk_pixbuf_new_from_inline (-1, default_icon_data,
+                                     FALSE,
+                                     NULL);
 
   g_assert (base);
 
@@ -2380,15 +2303,15 @@ _wnck_get_fallback_icons (GdkPixbuf **iconp,
 {
   if (iconp)
     *iconp = default_icon_at_size (ideal_width > 0 ? ideal_width :
-                                   (int) _wnck_get_default_icon_size (),
+                                   DEFAULT_ICON_WIDTH,
                                    ideal_height > 0 ? ideal_height :
-                                   (int) _wnck_get_default_icon_size ());
+                                   DEFAULT_ICON_HEIGHT);
 
   if (mini_iconp)
     *mini_iconp = default_icon_at_size (ideal_mini_width > 0 ? ideal_mini_width :
-                                        (int) _wnck_get_default_mini_icon_size (),
+                                        DEFAULT_MINI_ICON_WIDTH,
                                         ideal_mini_height > 0 ? ideal_mini_height :
-                                        (int) _wnck_get_default_mini_icon_size ());
+                                        DEFAULT_MINI_ICON_HEIGHT);
 }
 
 
